@@ -15,7 +15,7 @@ from jax._src.api import (_check_callable, _check_input_dtype_jacfwd, _check_inp
 
 
 def jvp_vmap(fun: Callable, argnums: Union[int, Sequence[int]] = 0):
-    """Vectorized (forward-mode) Jacobian-vector product of ``fun``. This is by large adopted from the implementation of jacfwd in jax._src.api.
+    """Vectorized (forward-mode) jacobian-vector product of ``fun``. This is by large adopted from the implementation of jacfwd in jax._src.api.
 
     Args:
       fun: Function whose value and Jacobian is to be computed.
@@ -29,7 +29,7 @@ def jvp_vmap(fun: Callable, argnums: Union[int, Sequence[int]] = 0):
     _check_callable(fun)
     argnums = _ensure_index(argnums)
 
-    def jacfun(args, tangents, **kwargs):
+    def jvpfun(args, tangents, **kwargs):
 
         f = lu.wrap_init(fun, kwargs)
         f_partial, dyn_args = argnums_partial(
@@ -39,7 +39,32 @@ def jvp_vmap(fun: Callable, argnums: Union[int, Sequence[int]] = 0):
 
         return y, jac
 
-    return jacfun
+    return jvpfun
+
+
+def vjp_vmap(fun: Callable, argnums: Union[int, Sequence[int]] = 0):
+    """Vectorized (reverse-mode) vector-jacobian product of ``fun``. This is by large adopted from the implementation of jacrev in jax._src.api.
+
+    Args:
+      fun: Function whose value and Jacobian are to be computed.
+      argnums: Optional, integer or sequence of integers. Specifies which
+        positional argument(s) to differentiate with respect to (default ``0``).
+
+    Returns:
+      A function with the same arguments as ``fun``, that evaluates the value and Jacobian of
+      ``fun`` using reverse-mode automatic differentiation.
+    """
+    _check_callable(fun)
+
+    def vjpfun(args, tangents, **kwargs):
+        f = lu.wrap_init(fun, kwargs)
+        f_partial, dyn_args = argnums_partial(f, argnums, args,
+                                              require_static_args_hashable=False)
+        y, pullback = _vjp(f_partial, *dyn_args)
+        jac = vmap(pullback)(tangents)
+        return y, jac
+
+    return vjpfun
 
 
 def jacfwd_and_val(fun: Callable, argnums: Union[int, Sequence[int]] = 0,
@@ -137,7 +162,7 @@ def jacrev_and_val(fun: Callable, argnums: Union[int, Sequence[int]] = 0,
     return jacfun
 
 
-def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=None, func_returns_jac=False, inspect_jac=False, verbose=False, verbose_jac=False):
+def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, rtol=None, sparse=False, solver=None, func_returns_jac=False, inspect_jac=False, verbose=False, verbose_jac=False):
     """Newton method for root finding using automatic differenciation with jax. The argument `func` must be jittable with jax.
 
     ...
@@ -176,6 +201,7 @@ def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=No
     st = time.time()
     verbose_jac |= inspect_jac
     verbose |= verbose_jac
+    rtol = rtol or tol
 
     if jac is None and not func_returns_jac:
         if sparse:
@@ -241,7 +267,7 @@ def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=No
             res['message'] = f"Maximum number of {maxit} iterations reached."
             break
 
-        if eps < tol:
+        if eps < rtol:
             res['success'] = True
             res['message'] = "The solution converged."
             break
@@ -255,8 +281,7 @@ def newton_jax(func, init, jac=None, maxit=30, tol=1e-8, sparse=False, solver=No
         jacval, (ssp._arrays.csr_array, ssp._arrays.lil_array)) else jacval
 
     res['x'], res['niter'] = xi, cnt
-    res['fun'] = func(xi)[0] if func_returns_jac else func(xi)
-    res['jac'] = jacval
+    res['fun'], res['jac'] = fval, jacval
 
     if verbose_jac:
         # only calculate determinant if requested
@@ -320,4 +345,5 @@ def newton_jax_jittable(func, init, jac=None, maxit=30, tol=1e-8):
     return tain[0], func(tain[0]), tain[2] < maxit, eps < tol
 
 
+amax = jax.jit(lambda x: jnp.abs(x).max())
 newton_jax_jit = jax.jit(newton_jax_jittable, static_argnums=(0, 2, 3, 4))
